@@ -1,6 +1,6 @@
 ;;; Support of miscellaneous kinds of speech events
 
-;; Copyright (C) 2003, 2004 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -148,23 +148,21 @@ EVENT-TYPE is one of the symbols `logical', `text', `sound', `key',
            (,mode-func ,mode-name))))))
 
 (define (event-find-seg-1 utt word placement)
-  (let ((neighbor ((if (eq? placement 'after) item.prev item.next) word)))
-    (cond
-     ((not neighbor)
-      (if (eq? placement 'after)
-          (list (utt.relation.first utt 'Segment) 'before)
-          (list (utt.relation.last utt 'Segment) 'after)))
-     ((not (string-equal
-            (item.feat neighbor "R:SylStructure.daughter1.daughter1.name")
-            0))
-      (let ((d (if (eq placement 'after) item.daughtern item.daughter1)))
-        (list (d (d (item.relation neighbor 'SylStructure))) placement)))
-     (t
-      (event-find-seg-1 utt neighbor placement)))))
+  (cond
+   ((not word)
+    (list ((if (eq? placement 'after) utt.relation.first utt.relation.last) utt 'Segment)
+          placement))
+   ((not (string-equal (item.feat word "R:SylStructure.daughter1.daughter1.name") 0))
+    (let ((d (if (eq placement 'after) item.daughtern item.daughter1)))
+      (list (d (d (item.relation word 'SylStructure))) placement)))
+   (t
+    (event-find-seg-1 utt ((if (eq? placement 'after) item.prev item.next) word) placement))))
 
 (define (event-find-seg utt word placement)
   (if (utt.relation.items utt 'Segment)
-      (event-find-seg-1 utt word placement)
+      (if (eq? placement 'after)
+          (event-find-seg-1 utt (item.next word) 'before)
+          (event-find-seg-1 utt (item.prev word) 'after))          
       (begin
         (utt.relation.append
          utt 'Segment (list (caar (cdar (PhoneSet.description '(silences))))))
@@ -173,34 +171,49 @@ EVENT-TYPE is one of the symbols `logical', `text', `sound', `key',
 (define (event-eat-utt utt wave-eater)
   (utt.relation.create utt 'Event)
   (do-relation-items (w utt Word)
-    (if (item.has_feat w 'event)
-        (let* ((placement* (if (string-equal (item.feat w 'event-stick-to)
-                                             'next)
-                               'before
-                               'after))
-               (seg-placement (event-find-seg utt w placement*))
-               (seg (first seg-placement))
-               (placement (second seg-placement)))
-          (item.set_feat seg 'event )
-          (item.set_feat seg 'event-placement placement)
-          (utt.relation.append utt 'Event
-                               `(event
-                                 ((event ,(item.feat w 'event))
-                                  (event-placement ,placement)
-                                  (end ,(item.feat seg 'end))
-                                  (pend ,(item.feat seg "R:Segment.p.end"))))
-                               ))))
+    (let* ((events '())
+           (get-event (lambda (item)
+                        (when (item.has_feat item 'event)
+                          (push (list (item.feat item 'event)
+                                      (if (string-equal (item.feat w 'event-stick-to) 'prev)
+                                          'before
+                                          'after))
+                                events)))))
+      (get-event w)
+      (let ((token (item.parent (item.relation w 'Token))))
+        (if (and token
+                 (or (not (item.next w))
+                     (not (equal? token (item.parent (item.relation (item.next w) 'Token))))))
+            (while token
+              (get-event token)
+              (set! token (item.next token))
+              (when (and token (item.daughters token))
+                (set! token nil)))))
+      (mapcar (lambda (event-direction)
+                (let* ((event (first event-direction))
+                       (direction (second event-direction))
+                       (seg-placement (event-find-seg utt w direction))
+                       (seg (first seg-placement))
+                       (placement (second seg-placement))
+                       (event* (utt.relation.append
+                                utt 'Event
+                                `(event ((event ,event)
+                                         (event-placement ,placement)
+                                         (end ,(item.feat seg 'end))
+                                         (pend ,(item.feat seg "R:Segment.p.end")))))))
+                  (item.set_feat seg 'event event*)))
+              (reverse events))))
   (let ((w (utt.wave utt)))
     (if (utt.relation.items utt 'Event)
         (let ((last-break 0.0))
-          (do-relation-items (seg utt Event)
-            (let ((break (if (string-equal (item.feat seg 'event-placement)
+          (do-relation-items (event utt Event)
+            (let ((break (if (string-equal (item.feat event 'event-placement)
                                            'after)
-                             (item.feat seg 'end)
-                             (or (item.feat seg 'pend) 0.0)))
-                  (event (item.feat seg 'event)))
+                             (item.feat event 'end)
+                             (or (item.feat event 'pend) 0.0)))
+                  (event* (item.feat event 'event)))
               (wave-eater (wave-subwave w last-break break))
-              (event-synth-plain (first event) (second event) wave-eater)
+              (event-synth-plain (first event*) (second event*) wave-eater)
               (set! last-break break)))
           (wave-eater (wave-subwave
                        w last-break
@@ -238,6 +251,8 @@ EVENT-TYPE is one of the symbols `logical', `text', `sound', `key',
                   value
                   (string-append sound-icon-directory "/" value)))))
     (wave-eater (utt.wave utt))
+    (when (string-matches value "^.*\.delete-after-play$")
+      (delete-file value))
     utt))
 
 (define (event-synth-plain type value wave-eater)
@@ -248,6 +263,8 @@ EVENT-TYPE is one of the symbols `logical', `text', `sound', `key',
     (event-synth-ssml value wave-eater))
    ((eq? type 'sound)
     (event-synth-sound value wave-eater))
+   ((eq? type 'mark)
+    (wave-eater (intern value)))
    (t
     (let ((transformed
            (cdr (assoc value (cadr (assq type (langvar 'event-mappings)))))))
