@@ -1,6 +1,6 @@
 ;;; Punctuation modes
 
-;; Copyright (C) 2003, 2004 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005, 2006 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -30,6 +30,8 @@
 (defvar punctuation-chars "[- !\"#$%&'()*+,./\\^_`:;<=>?@{|}~]")
 (defvar punctuation-chars-2 "[][]")
 
+(defvar punctuation-punc-languages '(english britishenglish americanenglish))
+
 ;; Default English voice doesn't have defined pronunciation of punctuation
 ;; characters
 (defvar punctuation-pronunciation
@@ -83,21 +85,35 @@
 
 (define (punctuation-process-words utt)
   (cond
-   ;; Standard English lexicon has no notion of punctuation pronounciation
-   ((and (eq? punctuation-mode 'all)
-         (member (Param.get 'Language)
-                 '(english britishenglish americanenglish
-                   "english" "britishenglish" "americanenglish")))
-    (do-relation-items (w utt Word)
-      (let ((trans (assoc (item.name w) punctuation-pronunciation)))
-        (if (and trans
-                 (not (word-mapping-of w)))
-            (begin
-              (item.set_name w (car (cdr trans)))
-              (set! trans (cdr (cdr trans)))
-              (while trans
-                (item.insert w (list (car trans)))
-                (set! trans (cdr trans))))))))
+   ((eq? punctuation-mode 'all)
+    (if (member (intern (Param.get 'Language)) punctuation-punc-languages)
+        ;; Standard English lexicon has no notion of punctuation pronounciation
+        (do-relation-items (w utt Word)
+          (let ((trans (assoc (item.name w) punctuation-pronunciation)))
+            (if (and trans
+                     (not (word-mapping-of w)))
+                (begin
+                  (item.set_name w (car (cdr trans)))
+                  (set! trans (cdr (cdr trans)))
+                  (while trans
+                    (let ((i (item.insert w (list (car trans)))))
+                      (item.append_daughter (item.parent (item.relation w 'Token))
+                                            i))
+                    (set! trans (cdr trans)))))))
+        ;; We assume other languages don't insert punctuation words themselves
+        (do-relation-items (w utt Word)
+          (let* ((w* (item.relation w 'Token))
+                 (token (item.parent w*)))
+            (when (and (not (item.prev w*))
+                       (item.has_feat token 'prepunctuation))
+              (dolist (p (reverse (symbolexplode (item.feat token 'prepunctuation))))
+                (let ((i (item.insert w `(,p ((name ,p))) 'before)))
+                  (item.prepend_daughter token i))))
+            (when (and (not (item.next w*))
+                       (item.has_feat token 'punc))
+              (dolist (p (reverse (symbolexplode (item.feat token 'punc))))
+                (let ((i (item.insert w `(,p ((name ,p))))))
+                  (item.append_daughter token i))))))))
    ;; Delete punctuation when punctuation-mode is none
    ((eq punctuation-mode 'none)
     (do-relation-top-items (token utt Token)
