@@ -1,6 +1,6 @@
 ;;; Miscellaneous utilities
 
-;; Copyright (C) 2003, 2004 Brailcom, o.p.s.
+;; Copyright (C) 2003, 2004, 2005 Brailcom, o.p.s.
 
 ;; Author: Milan Zamazal <pdm@brailcom.org>
 
@@ -67,6 +67,15 @@
 (define (third list)
   (nth 2 list))
 
+(define (fourth list)
+  (nth 3 list))
+
+(define (min x y)
+  (if (< x y) x y))
+
+(define (max x y)
+  (if (>= x y) x y))
+
 (define (remove-if test list)
   (let ((result '()))
     (while list
